import socket
import time
import json
from threading import Lock, Thread


class ClientUdpSocket:
    def __init__(self, ip, port,debug=False):
        """
        creates an UDP connection with specified host at a selected port
        :param ip: the ip that the udp wants to connect to.
        :param port: the port that the udp wants to connect to.
        :param debug: set to true if debug enabled.
        """
        self.socket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        self.data = None
        self.ip = ip
        self.port = port
        self.lock = Lock()
        self.shutdown = False
        self.new_data = False
        self.debug = debug

    def run(self):
        """
        runs the socket communication with the desierd server.

        :return: none.
        """
        contact = False
        timeout = 3
        self.socket.settimeout(timeout)
        while not contact:
            self.socket.sendto(b'h', (self.ip, self.port))
            try:
                data, addr = self.socket.recvfrom(2 ** 16)
            except socket.timeout:
                if self.debug == True:
                    print("Exception before connection")
                continue
            except ConnectionResetError:
                if self.debug == True:
                    print("No socket avalible!")
                continue

            # if no response: continue
            with self.lock:
                self.data = data
            contact = True
        while not self.shutdown:
            if self.debug == True:
                print('got contact')
            try:
                data, addr = self.socket.recvfrom(2 ** 16)
                with self.lock:
                    self.data = data
                    self.new_data = True
            except socket.timeout:
                if self.debug == True:
                    print("Exception after connection")

        self.socket.sendto(b'END', (self.ip, self.port))

    def runMap(self):
        """
        test function that is not in use.
        :return: none.
        """
        save = False
        start = False
        stop = False
        imData = []
        contact = False
        timeout = 3
        self.socket.settimeout(timeout)
        while not contact:
            self.socket.sendto(b'h', (self.ip, self.port))
            try:
                data, addr = self.socket.recvfrom(2 ** 16)
            except socket.timeout:
                if self.debug == True:
                    print("Exception before connection")
                continue
            except ConnectionResetError:
                if self.debug == True:
                    print("No socket avalible!")
                continue

            # if no response: continue
            contact = True
        while not self.shutdown:
            if self.debug == True:
                print('got contact')

            try:
                data, addr = self.socket.recvfrom(2 ** 16)

                if not start and data == b"start::":
                    print("start recived")
                    save = True
                    start = True
                    stop = False

                elif save and data == b"end::":
                    print("end recived")
                    with self.lock:
                        self.data = imData
                        self.new_data = True
                        save = False
                        stop = True
                        start = False
                        imData = []

                elif not stop and data == b"start::":
                    print("end packet lost!")
                    imData = []

                elif save == True:
                    packet = json.loads(data)

                    if type(packet) is list:
                        #for item in packet:
                            #imData.append(item)
                        imData.append(packet)

                    elif type(packet) is dict:
                        imData.append(packet)

                else:
                    imData = []


            except socket.timeout:
                if self.debug == True:
                    print("Exception after connection")

        self.socket.sendto(b'END', (self.ip, self.port))


    def retriveData(self):
        """
        retrives the data from the connected UDP server.

        :return: returns the retrived data from the UDP server.
        """
        data = None
        with self.lock:
            data = self.data
            self.new_data = False

        return data


if __name__ == '__main__':
    """
    test function to test the communcation.
    """
    client_socket = ClientUdpSocket('127.0.0.1', 5005)
    thread = Thread(target=client_socket.run)
    thread.start()

    for i in range(9):
        while not client_socket.new_data:
            pass
        print(client_socket.retriveData())

    client_socket.shutdown = True
    time.sleep(1)
